#!/usr/bin/env python
"""
    File rename utility.
    Created January 2012
    Copyright (C) Damien Farrell

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from __future__ import absolute_import, division, print_function
try:
    from tkinter import *
    from tkinter.ttk import *
    from tkinter.scrolledtext import ScrolledText
except:
    from Tkinter import *
    from ttk import *

import os, sys, string, time
if (sys.version_info > (3, 0)):
    from tkinter import filedialog, messagebox, simpledialog
else:
    import tkFileDialog as filedialog
    import tkSimpleDialog as simpledialog
    import tkMessageBox as messagebox
    from ScrolledText import ScrolledText
import re, glob
import pandas as pd
from pandastable.plugin import Plugin
from pandastable import Table, TableModel, dialogs
import pylab as plt

class MyTable(Table):
    """
      Custom table class inherits from Table.
      You can then override required methods
     """
    def __init__(self, parent=None, app=None, **kwargs):
        Table.__init__(self, parent, **kwargs)
        self.app = app
        return

    def handle_right_click(self, event):
        """respond to a right click"""

        return

    def handle_left_click(self, event):
        """respond to a right click"""

        Table.handle_left_click(self, event)
        self.app.previewTable()
        return

class BatchProcessPlugin(Plugin):
    """Batch processing plugin for DataExplore. Useful for multiple file import
    and plotting.
    """

    capabilities = ['gui','uses_sidepane']
    requires = ['']
    menuentry = 'Batch Process'
    gui_methods = {}
    version = '0.1'

    def __init__(self):
        return

    def main(self, parent):
        self.parent=parent
        if parent==None:
            self.main=Toplevel()
            self.master=self.main
            self.main.title('Batch Process')
            ws = self.main.winfo_screenwidth()
            hs = self.main.winfo_screenheight()
            w = 900; h=500
            x = (ws/2)-(w/2); y = (hs/2)-(h/2)
            self.main.geometry('%dx%d+%d+%d' % (w,h,x,y))
        else:
            self.parent = parent
            self._doFrame()
            self.main=self.mainwin
        self.doGUI()
        self.currentdir = self.homedir = os.path.expanduser('~')

        self.addFolder(path='test_batch')
        self.savepath = os.path.join(self.homedir,'batchplots')
        #self.test()
        return

    def doGUI(self):
        """Create GUI"""

        frame = Frame(self.main)
        frame.pack(side=LEFT,fill=BOTH,expand=1)
        df = pd.DataFrame()
        self.pt = MyTable(frame, app=self, dataframe=df, read_only=1, showtoolbar=0, width=100)
        self.pt.show()
        #self.m.add(frame)
        fr = Frame(self.main, padding=(4,4), width=120)
        fr.pack(side=RIGHT,fill=BOTH)

        b=Button(fr,text='Add Folder',command=self.addFolder)
        b.pack(side=TOP,fill=BOTH,pady=2)
        self.recursivevar = BooleanVar()
        self.recursivevar.set(False)
        b=Checkbutton(fr,text='Load Recursive',variable=self.recursivevar)
        b.pack(side=TOP,fill=BOTH,pady=2)
        b=Button(fr,text='Clear',command=self.clear)
        b.pack(side=TOP,fill=BOTH,pady=2)

        self.extensionvar = StringVar()
        w = Combobox(fr, values=['csv','tsv','txt','csv.gz','*'],
                 textvariable=self.extensionvar,width=6)
        w.pack(side=TOP,fill=BOTH,pady=2)
        w.set('csv')
        Label(fr,text='delimiter').pack()
        self.delimvar = StringVar()
        delimiters = [',',r'\t',' ',';','/','&','|','^','+','-']
        w = Combobox(fr, values=delimiters,
                 textvariable=self.delimvar,width=6)
        w.pack(side=TOP,fill=BOTH,pady=2)
        w.set(',')
        self.indexcolvar=IntVar()
        Label(fr,text='index column').pack()
        w=Entry(fr,textvariable=self.indexcolvar,width=6)
        w.pack(side=TOP,fill=BOTH,pady=2)
        self.useselectedvar = BooleanVar()
        self.useselectedvar.set(False)
        b=Checkbutton(fr,text='Selected Only',variable=self.useselectedvar)
        b.pack(side=TOP,fill=BOTH,pady=2)
        b=Button(fr,text='Import All',command=self.importAll)
        b.pack(side=TOP,fill=BOTH,pady=2)
        b=Button(fr,text='Preview Plot',command=self.previewPlot)
        b.pack(side=TOP,fill=BOTH,pady=2)
        b=Button(fr,text='Run Batch Plots',command=self.batchPlot)
        b.pack(side=TOP,fill=BOTH,pady=2)
        self.saveformatvar = StringVar()
        self.saveformatvar.set('png')
        w = Combobox(fr, values=['png','jpg','svg','tif','eps','pdf'],
                 textvariable=self.saveformatvar,width=6)
        w.pack(side=TOP,fill=BOTH,pady=2)
        b=Button(fr,text='Save Folder',command=self.selectSaveFolder)
        b.pack(side=TOP,fill=BOTH,pady=2)

        table = self.parent.getCurrentTable()
        self.pf = table.pf
        return

    def selectSaveFolder(self):
        self.savepath = filedialog.askdirectory(parent=self.main,
                                        initialdir=self.currentdir,
                                        title='Select folder')
        return

    def addFolder(self, path=None):
        """Get a folder"""

        if path==None:
            path = filedialog.askdirectory(parent=self.main,
                                            initialdir=self.currentdir,
                                            title='Select folder')
        if path:
            self.path = path
            self.refresh()
            self.currentdir = path
        return

    def refresh(self):
        """Load files list into table"""

        currdf = self.pt.model.df
        ext = self.extensionvar.get()
        fp = '*.'+ext
        if self.recursivevar.get() == 1:
            fp = '**/*.csv'
            flist = glob.glob(os.path.join(self.path,fp), recursive=True)
        else:
            flist = glob.glob(os.path.join(self.path,fp))
        sizes = []
        cols=[]; rows=[]
        for f in flist:
            s = os.path.getsize(f)
            sizes.append(s)
            df = pd.read_csv(f, nrows=10)
            cols.append(len(df.columns))

        df = pd.DataFrame({'filename':flist, 'filesize':sizes,
                            'columns':cols})
        #print (df)
        new = pd.concat([currdf,df])
        new = new.drop_duplicates('filename')
        self.pt.model.df = new
        self.pt.autoResizeColumns()
        self.pt.columnwidths['filename'] = 350
        #self.pt.redraw()
        return

    def loadFile(self, row=None):
        """Load a file from the table"""

        if row is None:
            row = self.pt.currentrow
        df = self.pt.model.df
        r = df.iloc[row]
        df = pd.read_csv(r.filename, sep=self.delimvar.get())
        idx=self.indexcolvar.get()
        df=df.set_index(df.columns[idx])
        return df

    def previewTable(self):
        """Preview selected table in main table."""

        df = self.loadFile()
        #print (df)
        table = self.parent.getCurrentTable()
        table.model.df = df
        table.autoResizeColumns()
        return

    def importAll(self):
        """Import selected or all files as tables"""

        ops = ['separately', 'concat', 'merge']
        d = dialogs.MultipleValDialog(title='Batch Import Files',
                                initialvalues=[ops],
                                labels=['How to Import:'],
                                types=['combobox'],
                                parent = self.mainwin)
        if d.result == None:
            return
        how = d.results[0]
        if how == 'join':
            self.joinTables()
        else:
            filelist = self.pt.model.df
            for i,r in filelist.iterrows():
                df = self.loadFile(i)
                self.parent.addSheet(i, df)
        return

    def joinTables(self):
        """Joins selected tables together and send to dataexplore"""

        filelist = self.pt.model.df
        res=[]
        for i,r in filelist.iterrows():
            df = self.loadFile(i)
            res.append(df)
        res = pd.concat(res)
        t = time.strftime('%X')
        self.parent.addSheet('joined-'+t, res)
        return

    def clear(self):
        """Clear file list"""

        self.path = None
        df = self.pt.model.df
        self.pt.model.df = df.iloc[0:0]
        self.pt.redraw()
        return

    def previewPlot(self):
        """Make preview plot"""

        cols = self.getCurrentSelections()
        df = self.loadFile()
        df = df[df.columns[cols]]
        self.pf.replot(df)
        return

    def batchPlot(self):
        """Plot multiple files"""

        plotdir = self.savepath
        if not os.path.exists(plotdir):
            os.mkdir(plotdir)
        format = self.saveformatvar.get()
        if format == 'pdf':
            pdf_pages = self.pdfPages()
        df = self.pt.model.df
        cols = self.getCurrentSelections()
        for i,r in df.iterrows():
            f = r.filename
            name = os.path.basename(f)
            df = self.loadFile(i)
            if len(df)==0:
                continue
            df = df[df.columns[cols]]
            self.pf.replot(df)
            self.pf.fig.suptitle(name)
            if format == 'pdf':
                fig = self.pf.fig
                pdf_pages.savefig(fig)
            else:
                self.pf.savePlot(filename=os.path.join(plotdir, name+'.'+format))
        if format == 'pdf':
            pdf_pages.close()
        return

    def getCurrentSelections(self):
        """Get row/col selections from main table for plotting"""

        table = self.parent.getCurrentTable()
        cols = table.multiplecollist
        return cols

    def pdfPages(self):
        """Create pdf pages object"""

        from matplotlib.backends.backend_pdf import PdfPages
        filename = os.path.join(self.savepath, 'batch_plots.pdf')
        pdf_pages = PdfPages(filename)
        from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
        return pdf_pages

    def test(self):
        path='test_batch'
        for i in range(20):
            df = TableModel.getSampleData()
            df.to_csv(os.path.join(path,'test%s.csv' %str(i)))
        return

def main():
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-d", "--dir", dest="directory",
                        help="Folder of raw files")

    opts, remainder = parser.parse_args()
    app = BatchProcessPlugin()
    if opts.directory != None:
        app.addFolder(opts.directory)
    app.mainloop()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
    DataExplore plugin for seaborn plotting.
    Created Oct 2015
    Copyright (C) Damien Farrell

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from __future__ import absolute_import, division, print_function
import os, datetime
from pandastable.plugin import Plugin
from pandastable import plotting, dialogs
try:
    from tkinter import *
    from tkinter.ttk import *
except:
    from Tkinter import *
    from ttk import *
import pandas as pd
import pylab as plt
from collections import OrderedDict
#from pandastable.dialogs import *

class DataReaderPlugin(Plugin):
    """Plugin for using pandas datareader for remote data access"""

    #capabilities = ['gui']
    requires = ['']
    menuentry = 'Remote Data'
    gui_methods = {}
    version = '0.1'

    def __init__(self):
        return

    def main(self, parent):

        if parent==None:
            return
        self.parent = parent
        self._doFrame(width=480,height=140)
        self.mainwin.title('Remote Data')
        self.mainwin.resizable(width=False,height=False)
        sources = ['Google Finance', 'FRED','World Bank','Eurostat','TSP','FAMA/French']
        dformats = ['%m/%d/%Y','%d/%m/%Y']
        grps = {'sources':['source','dataset','url'], 'time':['start','end','dateformat']}
        self.groups = grps = OrderedDict(grps)

        datacols = []
        self.opts = {
                     'source': {'type':'combobox','default':'FRED','items':sources,'width':20},
                     'dataset': {'type':'combobox','default':'F','items':['F']},
                     'url': {'type':'entry','default':'','label':'download url','width':25},
                     'start': {'type':'entry','default':'01/01/16','label':'start date','width':15},
                     'end': {'type':'entry','default':'31/12/16','label':'end date'},
                     'dateformat': {'type':'combobox','default':'%m/%d/%Y','items':dformats}
                     }
        fr = self._createWidgets(self.mainwin)
        fr.pack(side=LEFT,fill=BOTH)
        bf = Frame(self.mainwin, padding=2)
        bf.pack(side=LEFT,fill=BOTH)
        b = Button(bf, text="Fetch Data", command=self.fetch)
        b.pack(side=TOP,fill=X,pady=2)
        b = Button(bf, text="Close", command=self.quit)
        b.pack(side=TOP,fill=X,pady=2)
        b = Button(bf, text="About", command=self._aboutWindow)
        b.pack(side=TOP,fill=X,pady=2)
        #self.progframe = Frame(self.mainwin)
        #self.progframe.pack(side=TOP,fill=X)
        self.update()
        #self.prog_bar = Progress(bf)
        #self.prog_bar.pb_stop()
        return

    def fetch(self):

        import pandas_datareader as pdr
        import pandas_datareader.data as web
        from pandas_datareader import wb

        #self.prog_bar.pb_start()
        self.applyOptions()
        source = self.kwds['source']
        dataset = self.kwds['dataset']
        url = self.kwds['url']
        st = self.kwds['start']
        e = self.kwds['end']
        dateformat = self.kwds['dateformat']
        start = pd.to_datetime(st, format=dateformat, errors='ignore')
        end = pd.to_datetime(e, format=dateformat, errors='ignore')

        if url != '':
            df = self.fetch_url(url)
            if df is None:
                return
            label = os.path.basename(url)
            self.parent.load_dataframe(df, label, True)
            return

        if source == 'Yahoo Finance':
            df = web.DataReader("F", 'yahoo', start, end)
        elif source == 'Google Finance':
            df = web.DataReader("F", 'google', start, end)
        elif source == 'FRED':
            df = web.DataReader("GDP", "fred", start, end)
        elif source == 'OECD':
            df = web.DataReader('UN_DEN', 'oecd', end=end)
        elif source == 'World Bank':
            df = wb.download(indicator='NY.GDP.PCAP.KD', country=['US', 'CA', 'MX'], start=2005, end=2008)
        elif source == 'Eurostat':
            df = web.DataReader("tran_sf_railac", 'eurostat')
        elif source == 'TSP':
            import pandas_datareader.tsp as tsp
            tspreader = tsp.TSPReader(start=start, end=end)
            df = tspreader.read()
        elif source == 'FAMA/French':
            ds = web.DataReader(dataset, "famafrench")
            df = ds[0]
        label = source+'_'+dataset
        df=df.reset_index()
        self.parent.load_dataframe(df, label, True)
        #self.prog_bar.pb_stop()
        return

    def fetch_url(self, url):
        """Fetch data from an url"""

        ext = os.path.splitext(url)[1]
        if ext == '.zip':
            import zipfile, io, urllib
            r = urllib.request.urlopen(url)
            z = zipfile.ZipFile(io.BytesIO(r.read()))
            info = z.infolist()
            print (info)
            #for i in info:
            #    name = info.filename

        try:
            df = pd.read_csv(url)
        except Exception as e:
            messagebox.showwarning("Failed to read URL",
                                   "%s" %e,
                                   parent=self.parent)
            df=None
        return df

    def update(self, evt=None):
        """Update data widget(s)"""

        datasets = []
        self.applyOptions()
        source = self.kwds['source']
        x = ['F']
        if source == 'Yahoo Finance':
            x = ['F']
        elif source == 'FAMA/French':
            from pandas_datareader.famafrench import get_available_datasets
            x = get_available_datasets()

        w = self.widgets['dataset']
        w['values'] = x
        w.set(x[0])
        return

    def applyOptions(self):
        """Set the options"""

        kwds = {}
        for i in self.opts:
            if self.opts[i]['type'] == 'listbox':
                items = self.widgets[i].curselection()
                kwds[i] = [self.widgets[i].get(j) for j in items]
            else:
                kwds[i] = self.tkvars[i].get()
        self.kwds = kwds

        return

    def _createWidgets(self, parent, callback=None):
        """Auto create tk vars, widgets for corresponding options and
           and return the frame"""

        dialog, self.tkvars, self.widgets = plotting.dialogFromOptions(parent, self.opts, self.groups)
        self.widgets['source'].bind("<<ComboboxSelected>>", self.update)
        return dialog

    def quit(self, evt=None):
        """Override this to handle pane closing"""

        self.mainwin.destroy()
        return

    def about(self):
        """About this plugin"""

        txt = "This plugin allows fetching of remote data from\n"+\
              "multiple sources of public data using the pandas\n"+\
              "datareader. See https://pandas-datareader.readthedocs.io\n"+\
              "version: %s" %self.version
        return txt



#!/usr/bin/env python
"""
    Implements a configuration class for pandastable
    Created Oct 2015
    Copyright (C) Damien Farrell

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from __future__ import absolute_import, division, print_function
import math, time
import os, types
import string, copy
from collections import OrderedDict
try:
    from tkinter import *
    from tkinter.ttk import *
except:
    from Tkinter import *
    from ttk import *
try:
    import configparser
except:
    import ConfigParser as configparser
from . import util, plotting, dialogs

homepath = os.path.join(os.path.expanduser('~'))
configpath = os.path.join(homepath,'.config/pandastable')
if not os.path.exists(configpath):
    try:
        os.makedirs(configpath, exist_ok=True)
    except:
        os.makedirs(configpath)
default_conf = os.path.join(configpath, 'default.conf')

baseoptions = OrderedDict()
baseoptions['base'] = {'font': 'Arial','fontsize':12, 'fontstyle':'',
                        'floatprecision':2, 'thousandseparator': '',
                        'rowheight':22,'cellwidth':80, 'linewidth':1,
                        'align':'w',
                        }
baseoptions['colors'] =  {'cellbackgr':'#F4F4F3',
                        'textcolor':'black',
                        'grid_color':'#ABB1AD',
                        'rowselectedcolor':'#E4DED4',
                        'colheadercolor':'gray25'}

def write_default_config():
    """Write a default config to users .config folder. Used to add global settings."""

    fname = os.path.join(config_path, 'default.conf')
    if not os.path.exists(fname):
        try:
            #os.mkdir(config_path)
            os.makedirs(config_path)
        except:
            pass
        write_config(conffile=fname, defaults=baseoptions)
    return fname

def write_config(conffile='default.conf', defaults={}):
    """Write a default config file"""

    if not os.path.exists(conffile):
        cp = create_config_parser_from_dict(defaults)
        cp.write(open(conffile,'w'))
        print ('wrote config file %s' %conffile)
    return conffile

def create_config_parser_from_dict(data=None, sections=baseoptions.keys(), **kwargs):
    """Helper method to create a ConfigParser from a dict of the form shown in
       baseoptions"""

    if data is None:
        data = baseoptions
    #print (data)
    cp = configparser.ConfigParser()
    for s in sections:
        cp.add_section(s)
        if not s in data:
            continue
        for name in sorted(data[s]):
            val = data[s][name]
            if type(val) is list:
                val = ','.join(val)
            cp.set(s, name, str(val))

    #use kwargs to create specific settings in the appropriate section
    for s in cp.sections():
        opts = cp.options(s)
        for k in kwargs:
            if k in opts:
                cp.set(s, k, kwargs[k])
    return cp

def update_config(options):
    cp = create_config_parser_from_dict()
    for section in cp.sections():
        for o in cp[section]:
            cp[section][o] = str(options[o])
    return cp

def parse_config(conffile=None):
    """Parse a configparser file"""

    f = open(conffile,'r')
    cp = configparser.ConfigParser()
    try:
        cp.read(conffile)
    except Exception as e:
        print ('failed to read config file! check format')
        print ('Error returned:', e)
        return
    f.close()
    return cp

def get_options(cp):
    """Makes sure boolean opts are parsed"""

    from collections import OrderedDict
    options = OrderedDict()
    #options = cp._sections['base']
    for section in cp.sections():
        options.update( (cp._sections[section]) )
    for o in options:
        for section in cp.sections():
            try:
                options[o] = cp.getboolean(section, o)
            except:
                pass
            try:
                options[o] = cp.getint(section, o)
            except:
                pass
    return options

def print_options(options):
    """Print option key/value pairs"""

    for key in options:
        print (key, ':', options[key])
    print ()

def check_options(opts):
    """Check for missing default options in dict. Meant to handle
       incomplete config files"""

    sections = list(baseoptions.keys())
    for s in sections:
        defaults = dict(baseoptions[s])
        for i in defaults:
            if i not in opts:
                opts[i] = defaults[i]
    return opts

def load_options():
    if not os.path.exists(default_conf):
        write_config(default_conf, defaults=baseoptions)
    cp = parse_config(default_conf)
    options = get_options(cp)
    options = check_options(options)
    return options

def apply_options(options, table):
    """Apply options to a table"""

    for i in options:
        table.__dict__[i] = options[i]    
    table.setFont()
    table.redraw()
    return

class preferencesDialog(Frame):
    """Preferences dialog from config parser options"""

    def __init__(self, parent, options, table=None):

        self.parent = parent
        self.main = Toplevel()
        self.master = self.main
        x,y,w,h = dialogs.getParentGeometry(self.parent)
        self.main.geometry('+%d+%d' %(x+w/2-200,y+h/2-200))
        self.main.title('Preferences')
        self.main.protocol("WM_DELETE_WINDOW", self.quit)
        self.main.grab_set()
        self.main.transient(parent)
        self.main.resizable(width=False, height=False)
        self.createWidgets()
        self.updateFromOptions(options)
        self.options = options
        self.table = table
        return

    def createWidgets(self):
        """create widgets"""

        fonts = util.getFonts()

        self.opts = {'rowheight':{'type':'scale','default':18,'range':(5,50),'interval':1,'label':'row height'},
                'cellwidth':{'type':'scale','default':80,'range':(10,300),'interval':5,'label':'cell width'},
                'linewidth':{'type':'scale','default':1,'range':(1,10),'interval':1,'label':'grid line width'},
                'align':{'type':'combobox','default':'w','items':['w','e','center'],'label':'text align'},
                'vertlines':{'type':'checkbutton','default':1,'label':'show vertical lines'},
                'horizlines':{'type':'checkbutton','default':1,'label':'show horizontal lines'},
                'font':{'type':'combobox','default':'Arial','items':fonts},
                'fontstyle':{'type':'combobox','default':'','items':['','bold','italic']},
                'fontsize':{'type':'scale','default':12,'range':(5,40),'interval':1,'label':'font size'},
                'floatprecision':{'type':'entry','default':2,'label':'precision'},
                'thousandseparator':{'type':'combobox','default':'','items':['',','],'label':'thousands separator'},
                'cellbackgr':{'type':'colorchooser','default':'#F4F4F3', 'label':'background color'},
                'textcolor':{'type':'colorchooser','default':'black', 'label':'text color'},
                'grid_color':{'type':'colorchooser','default':'#ABB1AD', 'label':'grid color'},
                'rowselectedcolor':{'type':'colorchooser','default':'#E4DED4','label':'highlight color'},
                'colheadercolor':{'type':'colorchooser','default':'gray25','label':'column header color'},
                'colormap':{'type':'combobox','default':'Spectral','items':plotting.colormaps},
                'marker':{'type':'combobox','default':'','items':plotting.markers},
                'linestyle':{'type':'combobox','default':'-','items':plotting.linestyles},
                'ms':{'type':'scale','default':5,'range':(1,80),'interval':1,'label':'marker size'},
                'grid':{'type':'checkbutton','default':0,'label':'show grid'},
                }
        sections = {'table':['align','floatprecision','thousandseparator','rowheight','cellwidth','linewidth','vertlines','horizlines'],
                    'formats':['font','fontstyle','fontsize','cellbackgr','textcolor',
                               'grid_color','rowselectedcolor','colheadercolor']}
                    #'plotting':['marker','linestyle','ms','grid','colormap']}


        dialog, self.tkvars, self.widgets = dialogs.dialogFromOptions(self.main, self.opts, sections)
        dialog.pack(side=TOP,fill=BOTH)
        #d = dialogs.getDictfromTkVars(opts, tkvars, widgets)

        bf = Frame(self.main)
        bf.pack(fill=BOTH,expand=1)
        Button(bf, text='Apply', command=self.apply).pack(side=LEFT,padx=1,pady=1,fill=BOTH,expand=1)
        Button(bf, text='Save as Default',  command=self.save).pack(side=LEFT,padx=1,pady=1,fill=BOTH,expand=1)
        Button(bf, text='Close',  command=self.quit).pack(side=LEFT,padx=1,pady=1,fill=BOTH,expand=1)
        return

    def updateFromOptions(self, options):
        """Update all widget tk vars using dict"""

        if self.tkvars == None:
            return
        #print (options)
        for i in options:
            if i in self.tkvars and self.tkvars[i]:
                try:
                    val = int(options[i])
                except:
                    val = options[i]
                self.tkvars[i].set(val)
        return

    def apply(self):
        """Apply options to current table"""

        table = self.table
        options = dialogs.getDictfromTkVars(self.opts, self.tkvars, self.widgets)
        apply_options(options, table)
        return

    def save(self):
        """Save from current dialog settings"""

        options = dialogs.getDictfromTkVars(self.opts, self.tkvars, self.widgets)
        #print (options)
        #update configparser and write
        cp = update_config(options)
        cp.write(open(default_conf,'w'))
        return

    def quit(self):
        self.main.destroy()
        return



#!/usr/bin/env python
"""
    DataExplore pluin differential expression using R
    Created June 2017
    Copyright (C) Damien Farrell

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from __future__ import absolute_import, division, print_function
import sys,os
import subprocess
import numpy as np
from pandastable.plugin import Plugin
from pandastable import core, plotting, dialogs
try:
    from tkinter import *
    from tkinter.ttk import *
except:
    from Tkinter import *
    from ttk import *
import pandas as pd
import pylab as plt
from mpl_toolkits.mplot3d import Axes3D
from collections import OrderedDict

class MultivariatePlugin(Plugin):
    """Plugin for DataExplore"""

    capabilities = ['']#['gui','uses_sidepane']
    requires = ['']
    menuentry = 'Multivariate Analysis'
    gui_methods = {}
    version = '0.1'

    def __init__(self):
        self.result = None
        return

    def main(self, parent):

        if parent==None:
            return
        self.parent = parent
        self._doFrame()

        grps = {'data':['class_labels','target_col','use_selected'],
                'options':['analysis','transform','3d_plot']  }
        self.groups = grps = OrderedDict(grps)
        kinds = ['']
        methods = ['PCA','LDA','MDS','logistic_regression']#,'feature selection']
        transforms = ['','log']
        sheets = self.parent.getSheetList()
        self.opts = {'class_labels': {'type':'combobox','default':'','items':sheets},
                     'target_col': {'type':'combobox','default':'','items':[]},
                     'analysis': {'type':'combobox','default':'PCA','items':methods},
                     'use_selected': {'type':'checkbutton','default':False,'label':'use selected data'},
                     'transform': {'type':'combobox','default':'','items':transforms},
                     '3d_plot': {'type':'checkbutton','default':False,'label':'3d plot'},
                     }
        fr = self._createWidgets(self.mainwin)
        fr.pack(side=LEFT,fill=BOTH)

        bf = Frame(self.mainwin, padding=2)
        bf.pack(side=LEFT,fill=BOTH)

        b = Button(bf, text="Run", command=self.run)
        b.pack(side=TOP,fill=X,pady=2)
        b = Button(bf, text="View Results", command=self.showResults)
        b.pack(side=TOP,fill=X,pady=2)

        bf = Frame(self.mainwin, padding=2)
        bf.pack(side=LEFT,fill=BOTH)
        b = Button(bf, text="Refresh", command=self.update)
        b.pack(side=TOP,fill=X,pady=2)
        b = Button(bf, text="Close", command=self.quit)
        b.pack(side=TOP,fill=X,pady=2)
        b = Button(bf, text="Help", command=self.online_help)
        b.pack(side=TOP,fill=X,pady=2)

        self.update()
        sheet = self.parent.getCurrentSheet()
        #reference to parent frame in sheet
        pw = self.parent.sheetframes[sheet]
        self.pf = self.table.pf

        return

    def applyOptions(self):
        """Set the options"""

        kwds = {}
        for i in self.opts:
            if self.opts[i]['type'] == 'listbox':
                items = self.widgets[i].curselection()
                kwds[i] = [self.widgets[i].get(j) for j in items]
                print (items, kwds[i])
            else:
                kwds[i] = self.tkvars[i].get()
        self.kwds = kwds

        return

    def _createWidgets(self, parent, callback=None):
        """Auto create tk vars, widgets for corresponding options and
           and return the frame"""

        dialog, self.tkvars, self.widgets = plotting.dialogFromOptions(parent,
                                                                       self.opts, self.groups)
        #self.widgets['class_labels'].bind("<<ComboboxSelected>>", self.update)
        return dialog

    def update(self, evt=None):
        """Update data widget(s)"""

        self.table = self.parent.getCurrentTable()
        df = self.table.model.df
        cols = list(df.columns)
        cols += ''
        #self.widgets['sample_labels']['values'] = self.parent.getSheetList()
        self.widgets['class_labels']['values'] = cols
        self.widgets['target_col']['values'] = cols
        return

    def run(self):
        """Run chosen method"""

        import sklearn
        method = self.tkvars['analysis'].get()
        sel = self.tkvars['use_selected'].get()
        cats = self.tkvars['class_labels'].get()
        target = self.tkvars['target_col'].get()
        plot3d = self.tkvars['3d_plot'].get()
        transform = self.tkvars['transform'].get()

        if sel == 1:
            data = self.table.getSelectedDataFrame()
        else:
            data = self.table.model.df

        #setup plot
        self.pf._initFigure()
        if plot3d == True:
            fig = self.pf.fig
            ax = self.pf.ax = ax = Axes3D(fig)
        else:
            ax = self.pf.ax
        self.pf.mplopts.applyOptions()
        self.pf.labelopts.applyOptions()
        opts = self.pf.mplopts.kwds
        lopts = self.pf.labelopts.kwds
        #print (opts)

        if cats != '':
            X = data.set_index(cats)
        X = pre_process(data, transform=transform)
        print (X[:5])
        result = None

        if method == 'PCA':
            pX, result = do_pca(X=X)
            plot_matrix(pX, ax=ax, plot3d=plot3d, **opts)
        elif method == 'LDA':
            pX, result = do_lda(X=X)
            plot_matrix(pX, ax=ax, plot3d=plot3d, **opts)
        elif method == 'MDS':
            pX, result = do_mds(X=X)
            plot_matrix(pX, ax=ax, plot3d=plot3d, **opts)
        elif method == 'feature selection':
            pX = feature_selection(X)#, y=y)
        elif method == 'logistic_regression':
            pX = logistic_regression(X, ax, **opts)

        self.result_obj = result
        self.result_mat = pX
        self.pf.ax.set_title(lopts['title'])
        self.pf.canvas.draw()
        return

    def showResults(self):

        import sklearn
        df = self.result_mat
        result = self.result_obj
        if df is None:
            return
        w = self.resultswin = Toplevel(width=600,height=800)
        w.title('results')
        fr=Frame(w)
        fr.pack(fill=BOTH,expand=1)

        if type(result) is sklearn.decomposition.pca.PCA:
            print (result.components_)
        elif type(result) is sklearn.discriminant_analysis.LinearDiscriminantAnalysis:
            print (result)

        t = core.Table(fr, dataframe=df, showtoolbar=True)
        t.show()
        return

    def clustermap(self):

        data = self.data
        res = res = self.getFiltered()
        cluster_map(data, res.name)
        plt.show()
        return

    def quit(self, evt=None):
        """Override this to handle pane closing"""

        self.mainwin.destroy()
        return

    def online_help(self,event=None):
        """Open the online documentation"""
        import webbrowser
        link='https://github.com/dmnfarrell/pandastable/wiki'
        webbrowser.open(link,autoraise=1)
        return


def pre_process(X, transform='log'):

    X = X._get_numeric_data()
    if transform == 'log':
        X = X+1
        X = np.log(X)
    #print (X)
    X = X.fillna(0)
    return X

def do_pca(X, c=3):
    """Do PCA"""

    from sklearn import preprocessing
    from sklearn.decomposition.pca import PCA, RandomizedPCA
    #do PCA
    #S = standardize_data(X)
    #remove non numeric
    X = X._get_numeric_data()
    S = pd.DataFrame(preprocessing.scale(X),columns = X.columns)
    pca = PCA(n_components=c)
    pca.fit(S)
    out = 'explained variance %s' %pca.explained_variance_ratio_
    print (out)
    #print pca.components_
    w = pd.DataFrame(pca.components_,columns=S.columns)
    print (w.T.max(1).sort_values())
    pX = pca.fit_transform(S)
    pX = pd.DataFrame(pX,index=X.index)
    return pX, pca

def plot_matrix(pX, plot3d=False, palette='Spectral', labels=False, ax=None,
             colors=None, **kwargs):
    """Plot PCA result, input should be a dataframe"""

    if ax==None:
        fig,ax = plt.subplots(1,1,figsize=(6,6))
    #print (kwargs)
    colormap = kwargs['colormap']
    fs = kwargs['fontsize']
    ms = kwargs['ms']*12
    kwargs = {k:kwargs[k] for k in ('linewidth','alpha')}

    cats = pX.index.unique()
    import seaborn as sns
    colors = sns.mpl_palette(colormap, len(cats))

    for c, i in zip(colors, cats):
        print (i, len(pX.ix[i]))
        if plot3d == True:
            ax.scatter(pX.ix[i, 0], pX.ix[i, 1], pX.ix[i, 2], color=c, s=ms, label=i,
                        edgecolor='black', **kwargs)
        else:
            ax.scatter(pX.ix[i, 0], pX.ix[i, 1], color=c, s=ms, label=i,
                        edgecolor='black', **kwargs)

    ax.set_xlabel('PC1')
    ax.set_ylabel('PC2')
    if labels == True:
        for i, point in pX.iterrows():
            ax.text(point[0]+.3, point[1]+.3, str(i),fontsize=(9))
    if len(cats)<20:
        ax.legend(fontsize=fs*.8)
    return

def do_lda(X, c=3):

    from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
    idx = X.index
    cla = pd.Categorical(idx)
    y = cla.codes
    X = X._get_numeric_data()
    lda = LinearDiscriminantAnalysis(n_components=c)
    pX = lda.fit(X, y).transform(X)
    pX = pd.DataFrame(pX,index=idx)
    return pX, lda

def do_mds(X, c=3):
    """Do MDS"""

    X = X._get_numeric_data()
    from sklearn import manifold
    seed = np.random.RandomState(seed=3)
    mds = manifold.MDS(n_components=c, max_iter=500, eps=1e-9, random_state=seed,
                        n_jobs=1)
    pX = mds.fit(X.values).embedding_
    pX = pd.DataFrame(pX,index=X.index)
    return pX, mds

def feature_selection(X, y=None):
    """feature selection"""

    if y is None:
        idx = X.index
        cla = pd.Categorical(idx)
        y = cla.codes
    X = X._get_numeric_data()
    from sklearn.feature_selection import SelectKBest
    from sklearn.feature_selection import chi2
    pX = SelectKBest(chi2, k='all').fit_transform(X, y)
    pX.shape
    pX = pd.DataFrame(pX,index=X.index)
    return pX

def logistic_regression(X, ax, **kwargs):

    idx = X.index
    cla = pd.Categorical(idx)
    y = cla.codes
    from sklearn import linear_model
    logreg = linear_model.LogisticRegression(C=1e5)
    X = X.values
    X = X[:, :2]
    logreg.fit(X, y)
    h = .02
    cmap = plt.cm.get_cmap(kwargs['colormap'])
    kwargs = {k:kwargs[k] for k in ('linewidth','alpha')}

    x_min, x_max = X[:, 0].min() - .5, X[:, 0].max() + .5
    y_min, y_max = X[:, 1].min() - .5, X[:, 1].max() + .5
    xx, yy = np.meshgrid(np.arange(x_min, x_max, h), np.arange(y_min, y_max, h))
    Z = logreg.predict(np.c_[xx.ravel(), yy.ravel()])

    # Put the result into a color plot
    Z = Z.reshape(xx.shape)
    ax.pcolormesh(xx, yy, Z, cmap=cmap)
    Z = logreg.predict(np.c_[xx.ravel(), yy.ravel()])
    ax.scatter(X[:, 0], X[:, 1], c=y, edgecolors='k', cmap=plt.cm.Paired, **kwargs)

    return Z

def cluster_map(data, names):
    import seaborn as sns
    import pylab as plt
    data = data.ix[names]
    X = np.log(data).fillna(0)
    cg = sns.clustermap(X,cmap='RdYlBu',figsize=(8,9),lw=1,linecolor='gray')
    mt = plt.setp(cg.ax_heatmap.yaxis.get_majorticklabels(), rotation=0)
    plt.setp(cg.ax_heatmap.xaxis.get_majorticklabels(), rotation=90)
    cg.fig.subplots_adjust(right=.75)
    return cg



#!/usr/bin/env python
"""
    DataExplore Application based on pandastable.
    Created January 2014
    Copyright (C) Damien Farrell

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from __future__ import absolute_import, print_function
import sys, datetime, pickle, gzip
try:
    from tkinter import *
    from tkinter.ttk import *
    #import ttkbootstrap as ttk
    #from ttkbootstrap.constants import *
except:
    from Tkinter import *
    from ttk import *
if (sys.version_info > (3, 0)):
    from tkinter import filedialog, messagebox, simpledialog
else:
    import tkFileDialog as filedialog
    import tkSimpleDialog as simpledialog
    import tkMessageBox as messagebox

from collections import OrderedDict
import matplotlib
matplotlib.use('TkAgg')
import pandas as pd
import re, os, platform, time
from .core import Table
from .data import TableModel
#from .prefs import Preferences
from . import images, util, dialogs, plotting, config
from .dialogs import MultipleValDialog
from . import plugin

class DataExplore(Frame):
    """DataExplore application using pandastable widget.
        Args:
            parent: parent tkinter Frame, default None
            data: data, a pandas DataFrame
            projfile: path to a project file, opened on launch
            msgpack: path to a dataframe stored as msgpack, default None
    """

    def __init__(self, parent=None, data=None, projfile=None, msgpack=None):
        """Initialize the application. """

        self.parent=parent
        if not self.parent:
            Frame.__init__(self)
            self.main=self.master
        else:
            self.main=Toplevel()
            self.master=self.main

        if getattr(sys, 'frozen', False):
            #the application is frozen
            self.modulepath = os.path.dirname(sys.executable)
        else:
            self.modulepath = os.path.dirname(__file__)

        icon = os.path.join(self.modulepath,'dataexplore.gif')
        img = PhotoImage(file=icon)
        self.main.tk.call('wm', 'iconphoto', self.main._w, img)

        # Get platform into a variable
        self.currplatform = platform.system()
        self.setConfigDir()
        #if not hasattr(self,'defaultsavedir'):
        self.defaultsavedir = os.path.join(os.path.expanduser('~'))
        self.loadAppOptions()
        #start logging
        self.start_logging()

        self.main.title('DataExplore')
        self.createMenuBar()
        self.discoverPlugins()
        self.setupGUI()
        self.setStyles()
        self.clipboarddf = None
        self.projopen = False

        opts = {'layout':{'type':'checkbutton','default':'horizontal'}}
        #self.prefs = Prefs('.dataexplore', opts=opts)
        if data != None:
            self.data = data
            self.newProject(data)
        elif projfile != None:
            self.loadProject(projfile)
        elif msgpack != None:
            self.load_msgpack(msgpack)
        else:
            self.newProject()
        self.main.protocol('WM_DELETE_WINDOW',self.quit)
        self.main.lift()
        return

    def start_logging(self):
        import logging
        from .core import logfile
        logging.basicConfig(filename=logfile,format='%(asctime)s %(message)s')

    def setStyles(self):
        """Set theme and widget styles"""

        style = self.style = Style(self)
        available_themes = self.style.theme_names()
        plf = util.checkOS()
        if plf == 'linux':
            style.theme_use('default')
        elif plf == 'darwin':
            style.theme_use('clam')

        self.bg = bg = self.style.lookup('TLabel.label', 'background')
        style.configure('Horizontal.TScale', background=bg)
        #set common background style for all widgets because of color issues
        #if plf in ['linux','darwin']:
        #    self.option_add("*background", bg)
        dialogs.applyStyle(self.menu)
        return

    def setConfigDir(self):
        """Set up config folder"""

        homepath = os.path.join(os.path.expanduser('~'))
        path = '.dataexplore'
        self.configpath = os.path.join(homepath, path)
        self.pluginpath = os.path.join(self.configpath, 'plugins')
        if not os.path.exists(self.configpath):
            os.mkdir(self.configpath)
            os.makedirs(self.pluginpath)
        return

    def setupGUI(self):
        """Add all GUI elements"""

        self.m = PanedWindow(self.main, orient=HORIZONTAL)
        self.m.pack(fill=BOTH,expand=1)
        self.nb = Notebook(self.main)
        self.m.add(self.nb)
        self.setGeometry()
        return

    def createMenuBar(self):
        """Create the menu bar for the application. """

        self.menu = Menu(self.main)
        file_menu = Menu(self.menu,tearoff=0)
        #add recent first
        self.createRecentMenu(file_menu)
        filemenuitems = {'01New Project':{'cmd': self.newProject},
                    '02Open Project':{'cmd': lambda: self.loadProject(asksave=True)},
                    '03Close':{'cmd':self.closeProject},
                    '04Save':{'cmd':self.saveProject},
                    '05Save As':{'cmd':self.saveasProject},
                    '06sep':'',
                    '07Import CSV':{'cmd':self.importCSV},
                    '08Import HDF5':{'cmd':self.importHDF},
                    '09Import from URL':{'cmd':self.importURL},
                    '10Import Excel':{'cmd':self.importExcel},
                    '10Export CSV':{'cmd':self.exportCSV},
                    '11sep':'',
                    '12Quit':{'cmd':self.quit}}

        self.file_menu = self.createPulldown(self.menu, filemenuitems, var=file_menu)
        self.menu.add_cascade(label='File',menu=self.file_menu['var'])

        editmenuitems = {'01Undo Last Change':{'cmd': self.undo},
                        '02Copy Table':{'cmd': self.copyTable},
                        '03Find/Replace':{'cmd':self.findText},
                        '04Preferences':{'cmd': self.currentTablePrefs}
                        }
        self.edit_menu = self.createPulldown(self.menu, editmenuitems)
        self.menu.add_cascade(label='Edit',menu=self.edit_menu['var'])

        self.sheet_menu={'01Add Sheet':{'cmd': lambda: self.addSheet(select=True)},
                         '02Remove Sheet':{'cmd': lambda: self.deleteSheet(ask=True)},
                         '03Copy Sheet':{'cmd':self.copySheet},
                         '04Rename Sheet':{'cmd':self.renameSheet},
                         #'05Sheet Description':{'cmd':self.editSheetDescription}
                         }
        self.sheet_menu = self.createPulldown(self.menu,self.sheet_menu)
        self.menu.add_cascade(label='Sheet',menu=self.sheet_menu['var'])

        self.view_menu={'01Zoom In':{'cmd': lambda: self._call('zoomIn')},
                        '02Zoom Out':{'cmd': lambda: self._call('zoomOut')},
                        '03Wrap Columns':{'cmd': lambda: self._call('setWrap')},
                        '04sep':'',
                        '05Dark Theme':{'cmd': lambda: self._call('setTheme', name='dark')},
                        '06Bold Theme':{'cmd': lambda: self._call('setTheme', name='bold')},
                        '07Default Theme':{'cmd': lambda: self._call('setTheme', name='default')},
                        }
        self.view_menu = self.createPulldown(self.menu,self.view_menu)
        self.menu.add_cascade(label='View',menu=self.view_menu['var'])

        self.table_menu={'01Describe Table':{'cmd':self.describe},
                         '02Convert Column Names':{'cmd':lambda: self._call('convertColumnNames')},
                         '03Convert Numeric':{'cmd': lambda: self._call('convertNumeric')},
                         '04Clean Data': {'cmd': lambda: self._call('cleanData')},
                         '05Find Duplicates': {'cmd': lambda: self._call('findDuplicates')},
                         '06Correlation Matrix':{'cmd': lambda: self._call('corrMatrix')},
                         '07Concatenate Tables':{'cmd':self.concat},
                         '08Table to Text':{'cmd': lambda: self._call('showasText')},
                         '09Table Info':{'cmd': lambda: self._call('showInfo')},
                         '10sep':'',
                         '11Transform Values':{'cmd': lambda: self._call('transform')},
                         '12Group-Aggregate':{'cmd': lambda: self._call('aggregate')},
                         '13Cross Tabulation':{'cmd': lambda: self._call('crosstab')},
                         '14Merge/Concat Tables': {'cmd': lambda: self._call('doCombine')},
                         '15Pivot Table':{'cmd': lambda: self._call('pivot')},
                         '16Melt Table':{'cmd': lambda: self._call('melt')},
                         '17Time Series Resampling':{'cmd': lambda: self._call('resample')}
                        }
        self.table_menu = self.createPulldown(self.menu,self.table_menu)
        self.menu.add_cascade(label='Tools',menu=self.table_menu['var'])

        self.dataset_menu={'01Sample Data':{'cmd':self.sampleData},
                         '03Iris Data':{'cmd': lambda: self.getData('iris.csv')},
                         '03Tips Data':{'cmd': lambda: self.getData('tips.csv')},
                         '04Stacked Data':{'cmd':self.getStackedData},
                         '05Pima Diabetes':
                             {'cmd': lambda: self.getData('pima.csv')},
                         '06Titanic':
                             {'cmd': lambda: self.getData('titanic3.csv')},
                         '07miRNA expression':
                             {'cmd': lambda: self.getData('miRNA.csv')},
                         '08CO2 time series':
                             {'cmd': lambda: self.getData('co2-ppm-mauna-loa.csv')},
                         '09Zoo Dataset':
                             {'cmd': lambda: self.getData('zoo_dataset.csv')},
                         }
        self.dataset_menu = self.createPulldown(self.menu,self.dataset_menu)
        self.menu.add_cascade(label='Datasets',menu=self.dataset_menu['var'])

        self.plots_menu={'01Store plot':{'cmd':self.addPlot},
                         '02Clear plots':{'cmd':self.updatePlotsMenu},
                         '03PDF report':{'cmd':self.pdfReport},
                         '04sep':''}
        self.plots_menu = self.createPulldown(self.menu,self.plots_menu)
        self.menu.add_cascade(label='Plots',menu=self.plots_menu['var'])

        self.plugin_menu={'01Update Plugins':{'cmd':self.discoverPlugins},
                          '02Install Plugin':{'cmd':self.installPlugin},
                          '03sep':''}
        self.plugin_menu=self.createPulldown(self.menu,self.plugin_menu)
        self.menu.add_cascade(label='Plugins',menu=self.plugin_menu['var'])

        self.help_menu={'01Online Help':{'cmd':self.online_documentation},
                        '02View Error Log':{'cmd':self.showErrorLog},
                        '03About':{'cmd':self.about}}
        self.help_menu=self.createPulldown(self.menu,self.help_menu)
        self.menu.add_cascade(label='Help',menu=self.help_menu['var'])

        self.main.config(menu=self.menu)
        return

    def createRecentMenu(self, menu):
        """Recent projects menu"""

        from functools import partial
        recent = self.appoptions['recent']
        recentmenu = Menu(menu)
        menu.add_cascade(label="Open Recent", menu=recentmenu)
        for r in recent:
            recentmenu.add_command(label=r, command=partial(self.loadProject, r))
        return

    def bring_to_foreground(self, set_focus=False):
        self.main.deiconify()
        self.main.attributes('-topmost', True)
        self.main.after_idle(self.main.attributes, '-topmost', False)
        self.main.lift()

        if set_focus:
            #Looks like at least on Windows the following is required for the window
            #to also get focus (deiconify, ..., iconify, deiconify)
            import platform
            if platform.system() != "Linux":
                # http://stackoverflow.com/a/13867710/261181
                self.main.iconify()
                self.main.deiconify()
        return

    def getBestGeometry(self):
        """Calculate optimal geometry from screen size"""

        ws = self.main.winfo_screenwidth()
        hs = self.main.winfo_screenheight()
        if ws<1400:
            g = '%dx%d+%d+%d' % (ws,hs,0,0)
            self.w = ws
        else:
            self.w = w = ws/1.3; h = hs*0.7
            x = (ws/2)-(w/2); y = (hs/2)-(h/2)
            g = '%dx%d+%d+%d' % (w,h,x,y)
        return g

    def setGeometry(self):
        self.winsize = self.getBestGeometry()
        self.main.geometry(self.winsize)
        return

    def createPulldown(self, menu, dict, var=None):
        """Create pulldown menu, returns a dict.
        Args:
            menu: parent menu bar
            dict: dictionary of the form -
            {'01item name':{'cmd':function name, 'sc': shortcut key}}
            var: an already created menu
        """

        if var is None:
            var = Menu(menu,tearoff=0)
        dialogs.applyStyle(var)
        items = list(dict.keys())
        items.sort()
        for item in items:
            if item[-3:] == 'sep':
                var.add_separator()
            else:
                command = dict[item]['cmd']
                label = '%-25s' %(item[2:])
                if 'img' in dict[item]:
                    img = dict[item]['img']
                else:
                    img = None
                if 'sc' in dict[item]:
                    sc = dict[item]['sc']
                    #bind command
                    #self.main.bind(sc, command)
                else:
                    sc = None
                var.add('command', label=label, command=command, image=img,
                        compound="left")#, accelerator=sc)
        dict['var'] = var
        return dict

    def progressDialog(self):

        t = Toplevel(self)
        pb = Progressbar(t, mode="indeterminate")
        pb.pack(side="bottom", fill=X)
        t.title('Progress')
        t.transient(self)
        t.grab_set()
        t.resizable(width=False, height=False)
        return pb

    def currentTablePrefs(self):
        """Preferences dialog"""

        table = self.getCurrentTable()
        table.showPreferences()
        return

    def loadMeta(self, table, meta):
        """Load meta data for a sheet, this includes plot options and
        table selections"""

        tablesettings = meta['table']
        if 'childtable' in meta:
            childtable = meta['childtable']
            childsettings = meta['childselected']
        else:
            childtable = None
        #load plot options
        opts = {'mplopts': table.pf.mplopts,
                'mplopts3d': table.pf.mplopts3d,
                'labelopts': table.pf.labelopts
                }
        for m in opts:
            if m in meta and meta[m] is not None:
                #util.setAttributes(opts[m], meta[m])
                opts[m].updateFromDict(meta[m])
                #check options loaded for missing values
                #avoids breaking file saves when options changed
                defaults = plotting.get_defaults(m)
                for key in defaults:
                    if key not in opts[m].opts:
                        opts[m].opts[key] = defaults[key]

        #load table settings
        util.setAttributes(table, tablesettings)
        #load plotviewer
        if 'plotviewer' in meta:
            #print (meta['plotviewer'])
            util.setAttributes(table.pf, meta['plotviewer'])
            table.pf.updateWidgets()

        if childtable is not None:
            table.createChildTable(df=childtable)
            util.setAttributes(table.child, childsettings)

        #redraw col selections
        if type(table.multiplecollist) is tuple:
            table.multiplecollist = list(table.multiplecollist)
        table.drawMultipleCols()
        return

    def saveMeta(self, table):
        """Save meta data such as current plot options"""

        meta = {}
        #save plot options
        meta['mplopts'] = table.pf.mplopts.kwds
        meta['mplopts3d'] = table.pf.mplopts3d.kwds
        meta['labelopts'] = table.pf.labelopts.kwds
        #print (table.pf.mplopts.kwds)

        #save table selections
        meta['table'] = util.getAttributes(table)
        meta['plotviewer'] = util.getAttributes(table.pf)
        #print (meta['plotviewer'])
        #save row colors since its a dataframe and isn't picked up by getattributes currently
        meta['table']['rowcolors'] = table.rowcolors
        #save child table if present
        if table.child != None:
            meta['childtable'] = table.child.model.df
            meta['childselected'] = util.getAttributes(table.child)

        return meta

    def saveAppOptions(self):
        """Save global app options to config dir"""

        appfile = os.path.join(self.configpath, 'app.p')
        file = open(appfile,'wb')
        pickle.dump(self.appoptions, file, protocol=2)
        file.close()
        return

    def loadAppOptions(self):
        """Load global app options if present"""

        appfile = os.path.join(self.configpath, 'app.p')
        if os.path.exists(appfile):
            self.appoptions = pickle.load(open(appfile,'rb'))
        else:
            self.appoptions = {}
            self.appoptions['recent'] = []
        return

    def newProject(self, data=None, df=None):
        """Create a new project from data or empty"""

        w = self.closeProject()
        if w == None:
            return
        self.sheets = OrderedDict()
        self.sheetframes = {} #store references to enclosing widgets
        self.openplugins = {} #refs to running plugins
        self.updatePlotsMenu()
        for n in self.nb.tabs():
            self.nb.forget(n)
        if data != None:
            for s in sorted(data.keys()):
                if s == 'meta':
                    continue
                df = data[s]['table']
                if 'meta' in data[s]:
                    meta = data[s]['meta']
                else:
                    meta=None
                #try:
                self.addSheet(s, df, meta)
                '''except Exception as e:
                    print ('error reading in options?')
                    print (e)'''
        else:
            self.addSheet('sheet1')
        self.filename = None
        self.projopen = True
        self.main.title('DataExplore')
        return

    def loadProject(self, filename=None, asksave=False):
        """Open project file"""

        w=True
        if asksave == True:
            w = self.closeProject()
        if w == None:
            return

        if filename == None:
            filename = filedialog.askopenfilename(defaultextension='.dexpl"',
                                                    initialdir=self.defaultsavedir,
                                                    filetypes=[("project","*.dexpl"),
                                                               ("All files","*.*")],
                                                    parent=self.main)
        if not filename:
            return
        if not os.path.exists(filename):
            print ('no such file')
            self.removeRecent(filename)
            return
        ext = os.path.splitext(filename)[1]
        if ext != '.dexpl':
            print ('does not appear to be a project file')
            return
        if os.path.isfile(filename):
            #new format uses pickle
            try:
                data = pickle.load(gzip.GzipFile(filename, 'r'))
            except OSError as oe:
                msg = 'DataExplore can no longer open the old format project files.\n'\
                'if you really need the file revert to pandastable<=0.12.1 and save the data.'
                messagebox.showwarning("Project open error", msg)
                return
            #create backup file before we change anything
            #backupfile = filename+'.bak'
            #pd.to_msgpack(backupfile, data, encoding='utf-8')
        else:
            print ('no such file')
            self.quit()
            return
        self.newProject(data)
        self.filename = filename
        self.main.title('%s - DataExplore' %filename)
        self.projopen = True
        self.defaultsavedir = os.path.dirname(os.path.abspath(filename))
        self.addRecent(filename)
        return

    def removeRecent(self, filename):
        """Remove file from recent list"""

        recent = self.appoptions['recent']
        if filename in recent:
            recent.remove(filename)
            self.saveAppOptions()
        return

    def addRecent(self, filename):
        """Add file name to recent projects"""

        recent = self.appoptions['recent']
        if not os.path.abspath(filename) in recent:
            if len(recent)>=5:
                recent.pop(0)
            recent.append(os.path.abspath(filename))
            self.saveAppOptions()
        return

    def saveProject(self, filename=None):
        """Save project"""

        if filename != None:
            self.filename = filename
        if not hasattr(self, 'filename') or self.filename == None:
            self.saveasProject()
        else:
            self.doSaveProject(self.filename)
        return

    def saveasProject(self):
        """Save as a new filename"""

        filename = filedialog.asksaveasfilename(parent=self.main,
                                                defaultextension='.dexpl',
                                                initialdir=self.defaultsavedir,
                                                filetypes=[("project","*.dexpl")])
        if not filename:
            return
        self.filename = filename
        self.defaultsavedir = os.path.dirname(os.path.abspath(filename))
        self.doSaveProject(self.filename)
        self.addRecent(filename)
        return

    def doSaveProject(self, filename):
        """Save sheets as dict in msgpack"""

        self._checkTables()
        data={}
        for i in self.sheets:
            table = self.sheets[i]
            data[i] = {}
            data[i]['table'] = table.model.df
            data[i]['meta'] = self.saveMeta(table)

        #pd.to_msgpack(filename, data, encoding='utf-8')
        #changed to pickle format
        file = gzip.GzipFile(filename, 'w')
        pickle.dump(data, file)
        return

    def _checkTables(self):
        """Check tables before saving that so we are not saving
        filtered copies"""

        for s in self.sheets:
            t=self.sheets[s]
            if t.filtered==True:
                t.showAll()
        return

    def closeProject(self):
        """Close"""

        if self.projopen == False:
            w = False
        else:
            w = messagebox.askyesnocancel("Close Project",
                                        "Save this project?",
                                        parent=self.master)
        if w==None:
            return
        elif w==True:
            self.saveProject()
        else:
            pass
        for n in self.nb.tabs():
            self.nb.forget(n)
        self.filename = None
        self.projopen = False
        self.main.title('DataExplore')
        return w

    def importCSV(self):
        """Import csv to a new sheet"""

        self.addSheet(select=True)
        table = self.getCurrentTable()
        table.importCSV(dialog=True)
        return

    def importHDF(self):
        """Import csv to a new sheet"""

        self.addSheet(select=True)
        table = self.getCurrentTable()
        table.importHDF(dialog=True)
        return

    def importURL(self):
        """Import CSV from URL"""

        url = simpledialog.askstring("Import url", "Input CSV URL",
                                     parent=self.master)
        if url is not None:
            name = os.path.basename(url)
            df = pd.read_csv(url)
            self.addSheet(sheetname=name, df=df, select=True)
        return

    def exportCSV(self):
        """Import csv to a new sheet"""

        table = self.getCurrentTable()
        table.doExport()
        return

    def importExcel(self, filename=None):
        if filename is None:
            filename = filedialog.askopenfilename(parent=self.master,
                                                          defaultextension='.xls',
                                                          initialdir=os.getcwd(),
                                                          filetypes=[("xls","*.xls"),
                                                                     ("xlsx","*.xlsx"),
                                                            ("All files","*.*")])

        data = pd.read_excel(filename,sheet_name=None)
        for n in data:
            self.addSheet(n, df=data[n], select=True)
        return

    def load_dataframe(self, df, name=None, select=False):
        """Load a DataFrame into a new sheet
           Args:
            df: dataframe
            name: name of new sheet
            select: set new sheet as selected
        """

        if hasattr(self,'sheets'):
            self.addSheet(sheetname=name, df=df, select=select)
        else:
            data = {name:{'table':df}}
            self.newProject(data)
        return

    def load_msgpack(self, filename):
        """Load a msgpack file"""

        size = round((os.path.getsize(filename)/1.0485e6),2)
        print (size)
        df = pd.read_msgpack(filename)
        name = os.path.splitext(os.path.basename(filename))[0]
        self.load_dataframe(df, name)
        return

    def load_pickle(self, filename):
        """Load a pickle file"""

        df = pd.read_pickle(filename)
        name = os.path.splitext(os.path.basename(filename))[0]
        self.load_dataframe(df, name)
        return

    def getData(self, name):
        """Get predefined data from dataset folder"""

        filename = os.path.join(self.modulepath, 'datasets', name)
        df = pd.read_csv(filename, index_col=0)
        name = os.path.splitext(os.path.basename(filename))[0]
        self.load_dataframe(df, name, select=True)
        return

    def addSheet(self, sheetname=None, df=None, meta=None, select=False):
        """Add a sheet with new or existing data"""

        names = [self.nb.tab(i, "text") for i in self.nb.tabs()]
        def checkName(name):
            if name == '':
                messagebox.showwarning("Whoops", "Name should not be blank.")
                return 0
            if name in names:
                messagebox.showwarning("Name exists", "Sheet name already exists!")
                return 0

        noshts = len(self.nb.tabs())
        if sheetname == None:
            sheetname = simpledialog.askstring("New sheet name?", "Enter sheet name:",
                                                initialvalue='sheet'+str(noshts+1))
        if sheetname == None:
            return
        if checkName(sheetname) == 0:
            return
        #Create the table
        main = PanedWindow(orient=HORIZONTAL)
        self.sheetframes[sheetname] = main
        self.nb.add(main, text=sheetname)
        f1 = Frame(main)
        table = Table(f1, dataframe=df, showtoolbar=1, showstatusbar=1)
        f2 = Frame(main)
        #show the plot frame
        pf = table.showPlotViewer(f2)
        #load meta data
        if meta != None:
            self.loadMeta(table, meta)
        #add table last so we have save options loaded already
        main.add(f1,weight=3)
        table.show()
        main.add(f2,weight=4)

        if table.plotted == 'main':
            table.plotSelected()
        elif table.plotted == 'child' and table.child != None:
            table.child.plotSelected()
        self.saved = 0
        self.currenttable = table
        #attach menu state of undo item so that it's disabled after an undo
        #table.undo_callback = lambda: self.toggleUndoMenu('active')
        self.sheets[sheetname] = table

        if select == True:
            ind = self.nb.index('end')-1
            s = self.nb.tabs()[ind]
            self.nb.select(s)
        return sheetname

    def deleteSheet(self, ask=False):
        """Delete a sheet"""

        s = self.nb.index(self.nb.select())
        name = self.nb.tab(s, 'text')
        w=True
        if ask == True:
            w = messagebox.askyesno("Delete Sheet",
                                     "Remove this sheet?",
                                     parent=self.master)
        if w==False:
            return
        self.nb.forget(s)
        del self.sheets[name]
        del self.sheetframes[name]
        return

    def copySheet(self, newname=None):
        """Copy a sheet"""

        currenttable = self.getCurrentTable()
        newdata = currenttable.model.df
        meta = self.saveMeta(currenttable)
        self.addSheet(newname, df=newdata, meta=meta)
        return

    def renameSheet(self):
        """Rename a sheet"""

        s = self.nb.tab(self.nb.select(), 'text')
        newname = simpledialog.askstring("New sheet name?",
                                          "Enter new sheet name:",
                                          initialvalue=s)
        if newname == None:
            return
        self.copySheet(newname)
        self.deleteSheet()
        return

    def editSheetDescription(self):
        """Add some meta data about the sheet"""

        from .dialogs import SimpleEditor
        w = Toplevel(self.main)
        w.grab_set()
        w.transient(self)
        ed = SimpleEditor(w, height=25)
        ed.pack(in_=w, fill=BOTH, expand=Y)
        #ed.text.insert(END, buf.getvalue())
        return

    def getCurrentSheet(self):
        """Get current sheet name"""

        s = self.nb.index(self.nb.select())
        name = self.nb.tab(s, 'text')
        return name

    def getCurrentTable(self):

        s = self.nb.index(self.nb.select())
        name = self.nb.tab(s, 'text')
        table = self.sheets[name]
        return table

    def getSheetList(self):
        return list(self.sheets.keys())

    def describe(self):
        """Describe dataframe"""

        table = self.getCurrentTable()
        df = table.model.df
        d = df.describe()
        table.createChildTable(d,index=True)
        return

    def findText(self):

        table = self.getCurrentTable()
        table.findText()
        return

    def concat(self):
        """Concat 2 tables"""

        vals = list(self.sheets.keys())
        if len(vals)<=1:
            return
        d = MultipleValDialog(title='Concat',
                                initialvalues=(vals,vals),
                                labels=('Table 1','Table 2'),
                                types=('combobox','combobox'),
                                parent = self.master)
        if d.result == None:
            return
        else:
            s1 = d.results[0]
            s2 = d.results[1]
        if s1 == s2:
            return
        df1 = self.sheets[s1].model.df
        df2 = self.sheets[s2].model.df
        m = pd.concat([df1,df2])
        self.addSheet('concat-%s-%s' %(s1,s2),m)
        return

    def sampleData(self):
        """Load sample table"""

        d = MultipleValDialog(title='Sample Data',
                                initialvalues=(100,5),
                                labels=('Rows','Columns'),
                                types=('int','int'),
                                parent = self.master)
        if d.result == None:
            return
        rows=d.results[0]
        cols=d.results[1]
        df = TableModel.getSampleData(rows=rows,cols=cols)
        name='sample'
        i=1
        while name in self.sheets:
            name='sample'+str(i)
            i+=1
        self.addSheet(sheetname=name, df=df, select=True)
        return

    def getStackedData(self):

        df = TableModel.getStackedData()
        self.addSheet(sheetname='stacked-data', df=df)
        return

    def fileRename(self):
        """Start file renaming util"""

        from .rename import BatchRenameApp
        br = BatchRenameApp(self.master)
        return

    def copyTable(self, subtable=False):
        """Copy current table dataframe"""

        table = self.getCurrentTable()
        table.model.df.to_clipboard()
        return

    def pasteTable(self, subtable=False):
        """Paste copied dataframe into current table"""

        #add warning?
        if self.clipboarddf is None:
            return
        df = self.clipboarddf
        table = self.getCurrentTable()
        if subtable == True:
            table.createChildTable(df)
        else:
            model = TableModel(df)
            table.updateModel(model)
        return

    def discoverPlugins(self):
        """Discover available plugins"""

        if getattr(sys, 'frozen', False):
            #the application is frozen
            apppath = os.path.dirname(sys.executable)
        else:
            apppath = os.path.dirname(os.path.abspath(__file__))
        paths = [apppath,self.configpath]
        pluginpaths = [os.path.join(p, 'plugins') for p in paths]
        #print (pluginpaths)
        failed = plugin.init_plugin_system(pluginpaths)
        self.updatePluginMenu()
        return

    def installPlugin(self):
        """Adds a user supplied .py file to plugin folder"""

        filename = filedialog.askopenfilename(defaultextension='.py"',
                                              initialdir=os.getcwd(),
                                              filetypes=[("python","*.py")],
                                              parent=self.main)
        if filename:
            import shutil
            shutil.copy(filename, self.pluginpath)
            self.updatePluginMenu()
        return

    def updatePluginMenu(self):
        """Update plugins"""

        self.plugin_menu['var'].delete(3, self.plugin_menu['var'].index(END))
        plgmenu = self.plugin_menu['var']
        #for plg in plugin.get_plugins_instances('gui'):
        for plg in plugin.get_plugins_classes('gui'):
            def func(p, **kwargs):
                def new():
                   self.loadPlugin(p)
                return new
            plgmenu.add_command(label=plg.menuentry,
                               command=func(plg))
        return

    def loadPlugin(self, plugin):
        """Instantiate the plugin and call it's main method"""

        p = plugin()
        #plugin should add itself to the table frame if it's a dialog
        try:
            p.main(parent=self)
        except Exception as e:
            messagebox.showwarning("Plugin error", e,
                                    parent=self)
        name = self.getCurrentSheet()
        #track which plugin is running so the last one is removed?
        self.openplugins[name] = p
        return

    def hidePlot(self):
        name = self.getCurrentSheet()
        pw = self.sheetframes[name]
        pw.forget(1)
        return

    def showPlot(self):
        name = self.getCurrentSheet()
        table = self.sheets[name]
        pw = self.sheetframes[name]
        pw.add(table.pf, weight=2)
        return

    def addPlot(self):
        """Store the current plot so it can be re-loaded"""

        import pickle
        from . import plotting
        name = self.getCurrentSheet()
        table = self.sheets[name]
        fig = table.pf.fig
        t = time.strftime("%H:%M:%S")
        label = name+'-'+t
        #dump and reload the figure to get a new object
        p = pickle.dumps(fig)
        fig = pickle.loads(p)
        self.plots[label] = fig

        def func(label):
            fig = self.plots[label]
            win = Toplevel()
            win.title(label)
            plotting.addFigure(win, fig)

        menu = self.plots_menu['var']
        menu.add_command(label=label, command=lambda: func(label))
        return

    def updatePlotsMenu(self, clear=True):
        """Clear stored plots"""

        if clear == True:
            self.plots = {}
        menu = self.plots_menu['var']
        menu.delete(4, menu.index(END))
        return

    def pdfReport(self):
        """Create pdf report from stored plots"""

        from matplotlib.backends.backend_pdf import PdfPages
        filename = filedialog.asksaveasfilename(parent=self.main,
                                                defaultextension='.pdf',
                                                initialdir=self.defaultsavedir,
                                                filetypes=[("pdf","*.pdf")])
        if not filename:
            return
        pdf_pages = PdfPages(filename)
        from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

        for p in self.plots:
            fig = self.plots[p]
            canvas = FigureCanvasTkAgg(fig, master=self)
            pdf_pages.savefig(fig)
        pdf_pages.close()
        return

    def undo(self):
        """Restores last version of current table"""

        table = self.getCurrentTable()
        table.undo()
        #self.toggleUndoMenu('disabled')
        return

    def toggleUndoMenu(self, state='active'):
        menu = self.edit_menu['var']
        menu.entryconfigure(0, state=state)
        return

    def _call(self, func, **args):
        """Call a table function from it's string name"""

        table = self.getCurrentTable()
        getattr(table, func)(**args)
        return

    def _check_snap(self):
        if os.environ.has_key('SNAP_USER_COMMON'):
            print ('running inside snap')
            return True
        return False

    def about(self):
        """About dialog"""

        abwin = Toplevel()
        x,y,w,h = dialogs.getParentGeometry(self.main)
        abwin.geometry('+%d+%d' %(x+w/2-200,y+h/2-200))
        abwin.title('About')
        abwin.transient(self)
        abwin.grab_set()
        abwin.resizable(width=False, height=False)
        abwin.configure(background=self.bg)
        logo = images.tableapp_logo()
        label = Label(abwin,image=logo,anchor=CENTER)
        label.image = logo
        label.grid(row=0,column=0,sticky='ew',padx=4,pady=4)
        style = Style()
        style.configure("BW.TLabel", font='arial 11')
        from . import __version__
        pandasver = pd.__version__
        pythonver = platform.python_version()
        mplver = matplotlib.__version__
        if self._check_snap == True:
            snap='(snap)'
        else:
            snap=''

        text='DataExplore Application\n'\
                +'version '+__version__+snap+'\n'\
                +'Copyright (C) Damien Farrell 2014-\n'\
                +'This program is free software; you can redistribute it and/or\n'\
                +'modify it under the terms of the GNU General Public License\n'\
                +'as published by the Free Software Foundation; either version 3\n'\
                +'of the License, or (at your option) any later version.\n'\
                +'Using Python v%s\n' %pythonver\
                +'pandas v%s, matplotlib v%s' %(pandasver,mplver)

        row=1
        #for line in text:
        tmp = Label(abwin, text=text, style="BW.TLabel")
        tmp.grid(row=row,column=0,sticky='news',pady=2,padx=4)

        return

    def showErrorLog(self):
        """Open log file"""

        from .core import logfile
        f=open(logfile,'r')
        s=''.join(f.readlines())
        from .dialogs import SimpleEditor
        w = Toplevel(self)
        w.grab_set()
        w.transient(self)
        ed = SimpleEditor(w)
        ed.pack(in_=w, fill=BOTH, expand=Y)
        ed.text.insert(END, s)
        return

    def online_documentation(self,event=None):
        """Open the online documentation"""
        import webbrowser
        link='https://pandastable.readthedocs.io/en/latest/'
        webbrowser.open(link,autoraise=1)
        return

    def quit(self):
        self.main.destroy()
        return

class TestApp(Frame):
    """Basic test frame for the table"""
    def __init__(self, parent=None):
        self.parent = parent
        Frame.__init__(self)
        self.main = self.master
        self.main.geometry('800x500+200+100')
        self.main.title('DataExplore Test')
        f = Frame(self.main)
        f.pack(fill=BOTH,expand=1)
        df = TableModel.getSampleData()
        self.table = pt = Table(f, dataframe=df, enable_menus=True,
                                showtoolbar=True, showstatusbar=True)
        #options = config.load_options()
        options = {'floatprecision': 5, 'textcolor':'blue'}
        pt.show()
        #pt.hideRowHeader()
        config.apply_options(options, pt)
        self.table.rowheader.bgcolor = 'orange'
        self.table.colheader.bgcolor = 'lightgreen'
        self.table.colheader.textcolor = 'purple'
        #test row coloring
        pt.setRowColors(rows=range(2,100,2), clr='lightblue', cols='all')
        #test deleting
        pt.setSelectedRows([[4,6,8,10]])
        pt.deleteRow()
        pt.setSelectedRow()
        pt.insertRow()
        #pt.redraw()
        pt.editable = False
        #pt.showRowHeader()
        return

def main():
    "Run the application"
    import sys, os
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-f", "--file", dest="msgpack",
                        help="Open a dataframe as msgpack", metavar="FILE")
    parser.add_option("-p", "--project", dest="projfile",
                        help="Open a dataexplore project file", metavar="FILE")
    parser.add_option("-i", "--csv", dest="csv",
                        help="Import a csv file", metavar="FILE")
    parser.add_option("-x", "--excel", dest="excel",
                        help="Import an excel file", metavar="FILE")
    parser.add_option("-t", "--test", dest="test",  action="store_true",
                        default=False, help="Run a basic test app")

    opts, remainder = parser.parse_args()
    if opts.test == True:
        app = TestApp()
    else:
        if opts.projfile != None:
            if not os.path.exists(opts.projfile):
                print('no such file')
                return
            app = DataExplore(projfile=opts.projfile)
        elif opts.msgpack != None:
            app = DataExplore(msgpack=opts.msgpack)
        elif opts.csv != None:
            app = DataExplore()
            t = app.getCurrentTable()
            t.importCSV(opts.csv, dialog=True)
        elif opts.excel != None:
            app = DataExplore()
            app.importExcel(opts.excel)
        else:
            app = DataExplore()
    app.mainloop()
    return

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
    DataExplore pluin differential expression using R
    Created June 2017
    Copyright (C) Damien Farrell

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from __future__ import absolute_import, division, print_function
import sys,os
import subprocess
import numpy as np
from pandastable.plugin import Plugin
from pandastable import core, plotting, dialogs
try:
    from tkinter import *
    from tkinter.ttk import *
except:
    from Tkinter import *
    from ttk import *
import pandas as pd
import pylab as plt
from collections import OrderedDict

class ScriptRunnerPlugin(Plugin):
    """Plugin for DataExplore"""

    capabilities = ['gui','uses_sidepane']
    requires = ['']
    menuentry = 'Run Script'
    gui_methods = {}
    version = '0.1'

    def __init__(self):
        self.result = None
        return

    def main(self, parent):

        if parent==None:
            return
        self.parent = parent
        self._doFrame()
        w = self.mainwin
        fr = Frame(w)
        fr.pack(side=LEFT,fill=BOTH)
        self.ed = ed = dialogs.SimpleEditor(fr, width=60, height=25)
        ed.pack(in_=fr, fill=BOTH, expand=Y)
        ed.text.insert(END, '#run code here')

        bf = Frame(w, padding=2)
        bf.pack(side=LEFT,fill=BOTH)
        b = Button(bf, text="Run", command=self.run)
        b.pack(side=TOP,fill=X,pady=2)
        b = Button(bf, text="Close", command=self.quit)
        b.pack(side=TOP,fill=X,pady=2)
        b = Button(bf, text="Help", command=self.online_help)
        b.pack(side=TOP,fill=X,pady=2)

        #self.update()
        sheet = self.parent.getCurrentSheet()
        #reference to parent frame in sheet
        pw = self.parent.sheetframes[sheet]
        self.pf = self.table.pf

        return

    def run(self):
        """Run chosen method"""

        df = self.table.model.df
        safe_dict = {'df':df}

        code = self.ed.text.get(1.0, END).splitlines()
        for line in code:
            print (line)
            eval(line, {"__builtins__" : None }, safe_dict)

        return

    def createMethod(self, lines):
        def func(cls, x):
            return x
        setattr(self.__class__, 'func', func)

    def update(self):

        t = core.Table(fr, dataframe=df, showtoolbar=True)
        t.show()
        return

    def quit(self, evt=None):
        """Override this to handle pane closing"""

        self.mainwin.destroy()
        return

    def online_help(self,event=None):
        """Open the online documentation"""
        import webbrowser
        link='https://github.com/dmnfarrell/pandastable/wiki'
        webbrowser.open(link,autoraise=1)
        return


